package com.unissoft.common;

import com.alibaba.fastjson.JSON;
import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.systemManage.model.LogOperation;
import com.unissoft.systemManage.model.UserInfo;
import com.unissoft.systemManage.service.LogOperationService;
import com.unissoft.utils.IPUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

/**
 * @author qingsong.han
 * @description: 操作日志拦截器
 * @create 2021-10-21 15:00
 */
@Aspect
@Component
@SuppressWarnings("all")
public class LogAspect {
    /**
     * 操作版本号
     * <p>
     * 项目启动时从命令行传入，例如：java -jar xxx.war --version=201902
     * </p>
     */
    @Value("${version}")
    private String operVer;

    @Resource
    private LogOperationService logOperationService;

    /*
     * 设置操作日志切入点 记录操作日志 在注解的位置切入代码
     */
    @Pointcut("@annotation(com.unissoft.common.SystemLog)")
    public void pointcut() {
    }

    /**
     * 正常返回通知，拦截用户操作日志，连接点正常执行完成后执行， 如果连接点抛出异常，则不会执行
     *
     * @param joinPoint 切入点
     * @param keys      返回结果
     */
    @AfterReturning(value = "pointcut()", returning = "keys")
    public void saveOperLog(JoinPoint joinPoint, Object keys) {
        // 获取RequestAttributes
        RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
        // 从获取RequestAttributes中获取HttpServletRequest的信息
        HttpServletRequest request = (HttpServletRequest) requestAttributes
                .resolveReference(RequestAttributes.REFERENCE_REQUEST);

        LogOperation operlog = new LogOperation();
        try {
            operlog.setOperId(YitIdHelper.nextId()); // 主键ID

            // 从切面织入点处通过反射机制获取织入点处的方法
            MethodSignature signature = (MethodSignature) joinPoint.getSignature();
            // 获取切入点所在的方法
            Method method = signature.getMethod();
            // 获取操作
            SystemLog opLog = method.getAnnotation(SystemLog.class);
            if (opLog != null) {
                String operModul = opLog.operModul();
                String operType = opLog.operType();
                String operDesc = opLog.operDesc();
                operlog.setOperModul(operModul); // 操作模块
                operlog.setOperType(operType); // 操作类型
                operlog.setOperDesc(operDesc); // 操作描述
            }
            // 获取请求的类名
            String className = joinPoint.getTarget().getClass().getName();
            // 获取请求的方法名
            String methodName = method.getName();
            methodName = className + "." + methodName;

            operlog.setOperMethod(methodName); // 请求方法

            // 请求的参数
            Map<String, String> rtnMap = convertMap(request.getParameterMap());
            // 将参数所在的数组转换成json
            String params = JSON.toJSONString(rtnMap);

            operlog.setOperRequestParam(params); // 请求参数
            operlog.setOperResponsParam(JSON.toJSONString(keys)); // 返回结果
            UserInfo userInfo = (UserInfo) request.getSession().getAttribute("userLogin");
            if (userInfo != null) {
                operlog.setUserName(userInfo.getUserName()); // 请求用户ID
                operlog.setUserRealName(userInfo.getUserRealName()); // 请求用户名称
                operlog.setOrgCode(userInfo.getOrgCode()); // 请求用户组织机构编码
            } else {
                operlog.setUserName("admin"); // 请求用户ID
                operlog.setUserRealName("后台管理员"); // 请求用户名称
                operlog.setOrgCode("0"); // 请求用户组织机构编码
            }
            operlog.setOperIp(IPUtils.getRemoteIP(request)); // 请求IP
            operlog.setOperUri(request.getRequestURI()); // 请求URI
            operlog.setOperTime(DateUtils.getDate()); // 创建时间
            operlog.setOperVersion(operVer); // 操作版本
            logOperationService.save(operlog);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 转换request 请求参数
     *
     * @param paramMap request获取的参数数组
     */
    private Map<String, String> convertMap(Map<String, String[]> paramMap) {
        Map<String, String> rtnMap = new HashMap<>();
        for (String key : paramMap.keySet()) {
            rtnMap.put(key, paramMap.get(key)[0]);
        }
        return rtnMap;
    }

}
