package com.unissoft.config;

import com.unissoft.basic.service.RemoteParamsService;
import com.unissoft.mqtt.inbound.MqttMessageReceiver;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.core.annotation.Order;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.integration.annotation.ServiceActivator;
import org.springframework.integration.channel.DirectChannel;
import org.springframework.integration.config.EnableIntegration;
import org.springframework.integration.core.MessageProducer;
import org.springframework.integration.mqtt.core.MqttPahoClientFactory;
import org.springframework.integration.mqtt.inbound.MqttPahoMessageDrivenChannelAdapter;
import org.springframework.integration.mqtt.support.DefaultPahoMessageConverter;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.MessageHandler;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.Objects;

/**
 * @author qingsong.han
 * @description: 配置入站消息通道及相关处理Bean(消费订阅消息)
 * @create 2022-06-14 11:10
 */
@Slf4j
@Order(4)
@Component
@EnableIntegration
@ConditionalOnProperty(value = "mqtt.enabled", havingValue = "true")
public class MqttInConfig {
    /*
     * mqtt入站消息处理配置
     */
    @Value("${mqtt.topics}")
    private String topics;
    @Value("${mqtt.timeout}")
    private long timeout;
    @Value("${mqtt.qos}")
    private int qos;
    @Value("${mqtt.modelType}")
    private String modelType;
    @Value("${mqtt.clientId}")
    private String clientId;
    @Value("${mqtt.topicSN}")
    private String TOPIC_SN;
    @Value("${mqtt.topicType}")
    private String TOPIC_TYPE;

    private StringBuilder builder = new StringBuilder();
    private static final String TOPIC_PREFIX = "/PUSH/";
    private static final String TOPIC_POINT = "/DPUT/";

    @Resource
    private RemoteParamsService paramsService;
    @Resource
    private RedisTemplate redisTemplate;
    private final MqttMessageReceiver mqttMessageReceiver;

    @Autowired
    public MqttInConfig(MqttMessageReceiver mqttMessageReceiver) {
        this.mqttMessageReceiver = mqttMessageReceiver;
    }

    // 测试拼接topic
    //    private static final String TOPIC_TYPE = "upload,alarm,fault,set,time";
    /*@SuppressWarnings("all")
    public static void main(String[] args) {
        String topics = "mqtt_default_topic,2222";
        String topicSN = "8951DQ2207200031,8951DQ2207200034";
        String[] split = topicSN.split(",");
        StringBuilder builder = new StringBuilder();
        // 类型
        String[] split1 = TOPIC_TYPE.split(",");
        for (String s : split) {
            // /PUSH/${SN}/DPUT/
            String format = String.format("%s%s%s", TOPIC_PREFIX, s, TOPIC_POINT);
            for (String type : split1) {
                // /PUSH/${SN}/DPUT/upload
                builder.append(String.format("%s%s%s", format, type, ","));
            }
        }
        builder.append(topics);
        System.out.println(builder);
    }*/

    /**
     * 优先加载配置-bean 之前加载 使用动态配置文件方式使用(nacos,config)
     * /PUSH/${SN}/DPUT/upload
     */
    private void mqttInit() {
        if (StringUtils.isEmpty(topics) || StringUtils.isEmpty(TOPIC_SN))
            throw new IllegalArgumentException("mqtt topic is not null of topics or topicSN or topicType");
        /*动态拼接topics*/
        // 前缀+SN
        String[] split = TOPIC_SN.split(",");
        // 类型
        String[] split1 = TOPIC_TYPE.split(",");
        for (String s : split) {
            // /PUSH/${SN}/DPUT/
            String format = String.format("%s%s%s", TOPIC_PREFIX, s, TOPIC_POINT);
            for (String topicType : split1) {
                // /PUSH/${SN}/DPUT/upload
                builder.append(String.format("%s%s%s", format, topicType, ","));
            }
        }
        builder.append(topics);
        // 末位逗号(不去也可以在使用截取时会去除)
        topics = builder.toString();
    }


    /**
     * 数据库配置订阅消息
     */
    private void mqttInitDB() {
        mqttAddTopic(paramsService.getByTopics());
    }

    /**
     * redis 缓存读取订阅消息
     */
    private void mqttInitRedis() {
        mqttAddTopic((String) redisTemplate.opsForValue().get("mqtt_topics"));
    }

    /**
     * 添加配置topic
     *
     * @param newTopic 新topics
     */
    private void mqttAddTopic(String newTopic) {
        if (StringUtils.isEmpty(newTopic))
            builder.append("mqtt_default_topic");
        else
            builder.append(newTopic);
        topics = builder.toString();
    }

    /**
     * mqtt消息入站通道，订阅消息后消息进入的通道。
     * 可创建多个入站通道，对应多个不同的消息生产者。
     *
     * @return MessageChannel
     */
    @Bean
    public MessageChannel mqttInputChannel() {
        return new DirectChannel();
    }

    /**
     * 对于当前应用来讲，接收的mqtt消息的生产者。将生产者绑定到mqtt入站通道，即通过入站通道进入的消息为生产者生产的消息。
     * 可创建多个消息生产者，对应多个不同的消息入站通道，同时生产者监听不同的topic
     *
     * @return MessageProducer
     */
    @Bean
    public MessageProducer channelInbound(MessageChannel mqttInputChannel, MqttPahoClientFactory factory) {
        if (StringUtils.isEmpty(modelType)) {
            throw new IllegalArgumentException("mqtt.modelType config must set in object yaml");
        }
        if ("db_data".equals(modelType)) {
            this.mqttInitDB();
            log.info("------------------db_data MessageProducer start-------------------: {}", topics);
        } else if ("redis_data".equals(modelType)) {
            // redis缓存读取配置
            this.mqttInitRedis();
            log.info("------------------redis_data MessageProducer start-------------------: {}", topics);
        } else {
            // 默认配置文件方式
            this.mqttInit();
            log.info("------------------yaml config MessageProducer start-------------------: {}", topics);
        }
        String[] topic = topics.trim().split(",");
        MqttPahoMessageDrivenChannelAdapter adapter = new MqttPahoMessageDrivenChannelAdapter(clientId, factory, topic);
        adapter.setCompletionTimeout(timeout);
        adapter.setConverter(new DefaultPahoMessageConverter());
        adapter.setOutputChannel(mqttInputChannel);
        return adapter;
    }

    /**
     * mqtt入站消息处理工具，对于指定消息入站通道接收到生产者生产的消息后处理消息的工具。
     *
     * @return
     */
    @Bean
    @ServiceActivator(inputChannel = MqttBaseConfig.CHANNEL_NAME_IN)
    public MessageHandler mqttMessageHandler() {
        return this.mqttMessageReceiver;
    }

}
