package com.unissoft.deviceManage.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.yitter.idgen.YitIdHelper;
import com.google.common.collect.Lists;
import com.unissoft.common.DateUtils;
import com.unissoft.common.MyConstant;
import com.unissoft.common.PageParam;
import com.unissoft.deviceManage.mapper.DevicePointMapper;
import com.unissoft.deviceManage.model.*;
import com.unissoft.deviceManage.mapper.DeviceDataMapper;
import com.unissoft.deviceManage.service.DeviceDataService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.deviceManage.service.DeviceInfoService;
import com.unissoft.utils.RedisUtil;
import com.unissoft.visualisation.model.VisMonthWeek;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 设备检测结果(温湿度) 服务实现类
 * </p>
 *
 * @author root
 * @since 2022-07-07
 */
@Slf4j
@Service
public class DeviceDataServiceImpl extends ServiceImpl<DeviceDataMapper, DeviceData> implements DeviceDataService {
    @Resource
    private DeviceDataMapper deviceDataMapper;
    @Resource
    private RedisUtil redisUtil;
    @Resource
    private DeviceInfoService deviceInfoService;
    @Resource
    private DevicePointMapper devicePointMapper;

    /**
     * 温湿度设备数据转换
     *
     * @param deviceDataPoints 源数据集
     * @param SN               网关设备编码
     * @param cCode            数据类型:温湿度露点设备 200101
     */
    @SuppressWarnings("all")
    private DeviceData deviceDataConvert(List<DeviceDataPoint> deviceDataPoints) {
        // 过滤空对象
        deviceDataPoints = deviceDataPoints.stream().filter(d -> !Objects.isNull(d)).collect(Collectors.toList());
        // 创建数据对象
        DeviceData deviceData = new DeviceData();
        if (deviceDataPoints.size() > 0) {
            // 温度max,min,avg
            OptionalDouble maxOption = deviceDataPoints.stream().mapToDouble(DeviceDataPoint::getTempValue).max();
            if (maxOption.isPresent())
                deviceData.setMaxValue(maxOption.getAsDouble());
            OptionalDouble minOption = deviceDataPoints.stream().mapToDouble(DeviceDataPoint::getTempValue).min();
            if (minOption.isPresent())
                deviceData.setMinValue(minOption.getAsDouble());
            OptionalDouble avgOption = deviceDataPoints.stream().mapToDouble(DeviceDataPoint::getTempValue).average();
            if (avgOption.isPresent())
                deviceData.setAvgValue(Double.parseDouble(String.format("%.2f", avgOption.getAsDouble())));
            // 湿度max,min,avg
            OptionalDouble wetMax = deviceDataPoints.stream().mapToDouble(DeviceDataPoint::getWetValue).max();
            if (wetMax.isPresent())
                deviceData.setMaxWet(wetMax.getAsDouble());
            OptionalDouble wetMin = deviceDataPoints.stream().mapToDouble(DeviceDataPoint::getWetValue).min();
            if (wetMin.isPresent())
                deviceData.setMinWet(wetMin.getAsDouble());
            OptionalDouble wetAvg = deviceDataPoints.stream().mapToDouble(DeviceDataPoint::getWetValue).average();
            if (wetAvg.isPresent())
                deviceData.setAvgWet(Double.parseDouble(String.format("%.2f", wetAvg.getAsDouble())));
            // 露点max,min,avg
            OptionalDouble dewMax = deviceDataPoints.stream().mapToDouble(DeviceDataPoint::getDewValue).max();
            if (dewMax.isPresent())
                deviceData.setMaxDew(dewMax.getAsDouble());
            OptionalDouble dewMin = deviceDataPoints.stream().mapToDouble(DeviceDataPoint::getDewValue).min();
            if (dewMin.isPresent())
                deviceData.setMinDew(dewMin.getAsDouble());
            OptionalDouble dewAvg = deviceDataPoints.stream().mapToDouble(DeviceDataPoint::getDewValue).average();
            if (dewAvg.isPresent())
                deviceData.setAvgDew(Double.parseDouble(String.format("%.2f", dewAvg.getAsDouble())));
            // 保存到数据库
            deviceData.setId(YitIdHelper.nextId());
            // 设备编码
            deviceData.setEqCode(
                    deviceDataPoints.stream().map(DeviceDataPoint::getDevice).collect(Collectors.joining(","))
            );
            // 检测时间
            deviceData.setDataTime(new Date());
            // 数据json
            deviceData.setPoints(JSON.toJSONString(deviceDataPoints));
        }
        return deviceData;
    }

    @Transactional(rollbackFor = Exception.class)
    void beanConvert(String orgCode, String labelCode, DeviceDataPoint data) {
        DevicePoint devicePoint = new DevicePoint();
        if (!Objects.isNull(data)) {
            BeanUtils.copyProperties(data, devicePoint);
            devicePoint.setId(YitIdHelper.nextId());
            devicePoint.setTime(new Date());
            devicePoint.setOrgCode(orgCode);
            devicePoint.setLabelCode(labelCode);
            devicePointMapper.insert(devicePoint);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public DeviceData saveTempWet(CheckParams params) {
        // 200101 字典定义温湿度露点一体设备
        DeviceData deviceData = new DeviceData();
        if ("200101".equals(params.getProCategoryCode())) {
            // 初始化变量
            String redisData;
            DeviceDataPoint oneData = new DeviceDataPoint();
            ArrayList<DeviceDataPoint> points = Lists.newArrayList();
            // 根据orgCode获取组织下所有X1网关设备 (redis hash REDIS_X1_ORG设置orgCode:SN1,SN2...N)
            String orgCode = params.getOrgCode();
            // 支持多设备检测
            String eqCodeStr = params.getEqCode();
            // 支持按仓检测
            String labelCode = params.getLabelCode();
            // 非空
            if (Objects.nonNull(orgCode) && Objects.nonNull(labelCode)) {
                // 机构编码获取产品编码(查询redis缓存hash key)和设备编码(获取redis hash key value)
                List<DeviceInfo> dataInfo = deviceInfoService.proCodeFromOrgCodeAndLabelCode(orgCode, labelCode);
                if (dataInfo.size() > 0) {
                    // 判断产品编码是否唯一(一个区域是否安装一个网关设备)
                    Map<String, List<DeviceInfo>> proCodeGroup = dataInfo.stream().collect(Collectors.groupingBy(DeviceInfo::getProCode));
                    // 设备编码获取设备标签,接收到的设备编码必须同属一个设备标签
                    Set<String> redisKeys = proCodeGroup.keySet();
                    for (String proCode : redisKeys) {
                        // 判断redis hash 网关SN编码是否存在,存在才执行对应网关设备获取数据
                        if (redisUtil.hasKey(proCode)) {
                            // 获取多设备数据
                            if (Objects.nonNull(eqCodeStr)) {
                                String[] eqCodes = eqCodeStr.split(",");
                                for (String eq : eqCodes) {
                                    if (redisUtil.hHasKey(proCode, eq)) {
                                        // 单对象设备数据
                                        redisData = JSON.toJSONString(redisUtil.hget(proCode, eq));
                                        oneData = JSON.parseObject(redisData, DeviceDataPoint.class);
                                        points.add(oneData);
                                        // 保存单点设备数据
                                        this.beanConvert(orgCode, labelCode, oneData);
                                    }
                                }
                            } else { // 获取产品下的所有数据
                                // 获取hash key 下的所有数据
                                Map<String, Object> hmget = redisUtil.hmget(proCode);
                                for (String key : hmget.keySet()) {
                                    if (redisUtil.hHasKey(proCode, key)) {
                                        redisData = JSON.toJSONString(redisUtil.hget(proCode, key));
                                        oneData = JSON.parseObject(redisData, DeviceDataPoint.class);
                                        points.add(oneData);
                                    }
                                    // 保存单点设备数据
                                    this.beanConvert(orgCode, labelCode, oneData);
                                }
                            }
                        }
                    }
                    // 数据处理
                    deviceData = deviceDataConvert(points);
                    // 分布式部署需要判断
                    if (Objects.nonNull(deviceData.getEqCode())) {
                        // 数据类型温湿度(字典类型)
                        deviceData.setDataType(params.getProCategoryCode());
                        // orgCode
                        deviceData.setOrgCode(orgCode);
                        // labelCode==storeHouse
                        deviceData.setLabelCode(labelCode);
                        deviceData.setProCode(redisKeys.toString());
                        // 4.持久化
                        log.info("DeviceDataServiceImpl.class.saveTempWet: {}", JSON.toJSONString(deviceData));
                        deviceDataMapper.insert(deviceData);
                    }
                    return deviceData;
                }
            } else {
                throw new IllegalArgumentException("orgCode and labelCode are can not null");
            }
        } else {
            throw new IllegalArgumentException("proCategoryCode is can not null");
        }
        return deviceData;
    }

    @Override
    public IPage<DeviceData> getPageList(PageParam pageParam) {
        // 分页条件
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();

        // 查询对象
        QueryWrapper<DeviceData> wrapper = new QueryWrapper<>();
        String condition = pageParam.getCondition();
        if (!StringUtils.isEmpty(condition)) {
            DeviceData deviceData = JSON.parseObject(condition, DeviceData.class);
            Date dataTime = deviceData.getDataTime();
            String orgCode = deviceData.getOrgCode();
            String proCode = deviceData.getProCode();
            if (!Objects.isNull(dataTime)) {
                Date date = DateUtils.getDate(dataTime.getTime() + MyConstant.ONE_DAY - 1);
                wrapper.between("t2.data_time", dataTime, date);
            }
            if (!StringUtils.isEmpty(orgCode))
                wrapper.eq("t2.org_code", orgCode);
            if (!StringUtils.isEmpty(orgCode))
                wrapper.like("t2.pro_code", proCode);
        }
        // 排序条件
        wrapper.orderByDesc("t2.id");
        // 分页对象
        IPage<DeviceData> page = new Page<>(pageIndex, pageSize);
        return deviceDataMapper.selectPageList(page, wrapper);
    }

    @Override
    public IPage<DevicePoint> getPintsPageList(PageParam pageParam) {
        // 分页条件
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();

        // 查询对象
        QueryWrapper<DevicePoint> wrapper = new QueryWrapper<>();
        String condition = pageParam.getCondition();
        if (!StringUtils.isEmpty(condition)) {
            DevicePoint devicePoint = JSON.parseObject(condition, DevicePoint.class);
            Date dataTime = devicePoint.getTime();
            if (!StringUtils.isEmpty(dataTime)) {
                String endTime = String.format("%s %s", DateUtils.formatDate(dataTime), MyConstant.END_FLG);
                wrapper.between("dev_time", dataTime, endTime);
            }
            String orgCode = devicePoint.getOrgCode();
            if (!StringUtils.isEmpty(orgCode))
                wrapper.eq("org_code", orgCode);
            String device = devicePoint.getDevice();
            if (!StringUtils.isEmpty(device))
                wrapper.eq("deviceAlarm", device);
            String labelCode = devicePoint.getLabelCode();
            if (!StringUtils.isEmpty(labelCode))
                wrapper.eq("label_code", labelCode);
        }
        // 排序条件
        wrapper.orderByDesc("id");
        // 分页对象
        IPage<DevicePoint> page = new Page<>(pageIndex, pageSize);
        return devicePointMapper.selectPage(page, wrapper);
    }

    @Override
    public List<DeviceRunData> getDeviceRunData(Long id) {
        // 获取单个设备下多点数据
        List<DeviceJoinData> deviceSource = deviceDataMapper.getDeviceRunData(id);
        if (!CollectionUtils.isEmpty(deviceSource)) {
            // 获取对应的points(list中任意一条)
            String points = deviceSource.get(0).getPoints();
            // 转换数据处理对象
            if (!StringUtils.isEmpty(points)) {
                List<DeviceRunData> runData = JSONArray.parseArray(points, DeviceRunData.class);
                // 2个list比较合并参数
                runData.forEach(data -> deviceSource.forEach(ds -> {
                            if (data.getDevice().equals(ds.getEqCode())) {
                                // 填充参数
                                data.setEqName(ds.getEqName());
                                data.setLabelName(ds.getLabelName());
                            }
                        })
                );
                return runData;
            }
        }
        return Collections.emptyList();
    }

    @Override
    public List<DeviceData> initPointsDraw(String orgCode, String labelCode, Integer pageSize) {
        return deviceDataMapper.initPointsDraw(orgCode, labelCode, pageSize);
    }

    @Override
    public List<VisMonthWeek> monthAndWeek(String orgCode, String labelCode, String begin, String end) {
        return deviceDataMapper.monthAndWeek(orgCode, labelCode, begin, end);
    }

}
