package com.unissoft.deviceManage.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.yitter.idgen.YitIdHelper;
import com.unissoft.common.DateUtils;
import com.unissoft.common.MyConstant;
import com.unissoft.common.PageParam;
import com.unissoft.common.ResultView;
import com.unissoft.deviceManage.mapper.DeviceInfoMapper;
import com.unissoft.deviceManage.mapper.DeviceLabelMapper;
import com.unissoft.deviceManage.model.DeviceInfo;
import com.unissoft.deviceManage.model.DeviceInfoJoinProduct;
import com.unissoft.deviceManage.model.DeviceLabel;
import com.unissoft.deviceManage.service.DeviceInfoService;

import com.unissoft.productManage.mapper.ProductInfoMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;
import javax.validation.constraints.NotBlank;

/**
 * <p>
 * 设备信息表 服务实现类
 * </p>
 *
 * @author root
 * @since 2022-06-16
 */
@Service
public class DeviceInfoServiceImpl extends ServiceImpl<DeviceInfoMapper, DeviceInfo> implements DeviceInfoService {

    @Resource
    private DeviceInfoMapper deviceInfoMapper;
    @Resource
    private DeviceLabelMapper deviceLabelMapper;
    @Resource
    private ProductInfoMapper productInfoMapper;

    @Override
    public IPage<DeviceInfo> getPageList(PageParam pageParam) {
        // 分页条件
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();

        // 查询条件
        QueryWrapper<DeviceInfo> queryWrapper = new QueryWrapper<>();
        String condition = pageParam.getCondition();
        if (!StringUtils.isEmpty(condition)) {
            DeviceInfo deviceInfo = JSON.parseObject(condition, DeviceInfo.class);
            // 获取条件
            String orgCode = deviceInfo.getOrgCode();
            String proCode = deviceInfo.getProCode();
            String eqCode = deviceInfo.getEqCode();
            String labelCode = deviceInfo.getLabelCode();
            if (!StringUtils.isEmpty(orgCode))
                queryWrapper.eq("di.org_code", orgCode);
            if (!StringUtils.isEmpty(proCode))
                queryWrapper.eq("di.pro_code", proCode);
            if (!StringUtils.isEmpty(eqCode))
                queryWrapper.eq("di.eq_code", eqCode);
            if (!StringUtils.isEmpty(labelCode))
                queryWrapper.eq("di.label_code", labelCode);
        }
        // 排序条件
        queryWrapper.orderByDesc("di.register_time");
        // 分页对象
        IPage<DeviceInfo> page = new Page<>(pageIndex, pageSize);
        return deviceInfoMapper.selectPageList(page, queryWrapper);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveAndUpdate(DeviceInfo deviceInfo) {
        Date date = DateUtils.getDate();
        if (deviceInfo.getId() == null) {
            deviceInfo.setId(YitIdHelper.nextId());
            deviceInfo.setRegisterTime(date);
            // set默认设备编码
            if (StringUtils.isEmpty(deviceInfo.getEqCode())) {
                deviceInfo.setEqCode(String.valueOf(YitIdHelper.nextId()));
            }
            // 新增报错设备信息
            deviceInfoMapper.insert(deviceInfo);

            // 更改设备标签的删除标志
            UpdateWrapper<DeviceLabel> updateWrapper = new UpdateWrapper<>();
            updateWrapper.eq("label_code", deviceInfo.getLabelCode());
            updateWrapper.set("del_flag", 1);//不可删除
            deviceLabelMapper.update(null, updateWrapper);

            // 更新所属产品设备数量
            productInfoMapper.updateEqTotal(deviceInfo.getOrgCode(), deviceInfo.getProCode());
        } else {
            deviceInfo.setLastOnlineTime(date);
            deviceInfoMapper.updateById(deviceInfo);
        }
    }

    @Override
    public ResultView deleteById(Long id) {
        DeviceInfo deviceInfo = deviceInfoMapper.selectById(id);
        if (deviceInfo != null) {
            QueryWrapper<DeviceInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("label_code", deviceInfo.getLabelCode());
            int deviceListSize = deviceInfoMapper.selectList(queryWrapper).size();
            if (!(deviceListSize > 1)) { // 有自身的一条数据 所以大于1,否则不用处理
                UpdateWrapper<DeviceLabel> updateWrapper = new UpdateWrapper<>();
                updateWrapper.eq("label_code", deviceInfo.getLabelCode());
                updateWrapper.set("del_flag", 0);//可删除
                deviceLabelMapper.update(null, updateWrapper);
            }
            // 执行删除
            deviceInfoMapper.deleteById(id);

            // 更新所属产品设备数量
            productInfoMapper.updateEqTotal(deviceInfo.getOrgCode(), deviceInfo.getProCode());
            return ResultView.success();
        }
        return ResultView.error(MyConstant.ID_NULL);
    }

    @Override
    public DeviceInfo queryById(Long id) {
        // TODO Auto-generated method stub
        return deviceInfoMapper.queryById(id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void disassociateProduct(Long id) {
        // 更新产品设备数量
        productInfoMapper.updateEqTotalByDevId(id);
        // 更新关联
        deviceInfoMapper.updateProCodeById(id);
    }

    @Override
    public void addAssociationsProduct(DeviceInfoJoinProduct deviceInfoJoinProduct) {
        // 设备信息数据集
        List<DeviceInfo> deviceInfoList = deviceInfoJoinProduct.getDeviceInfoList();
        // 关联产品编码
        @NotBlank String proCode = deviceInfoJoinProduct.getProCode();
        // 批量更新
        if (!CollectionUtils.isEmpty(deviceInfoList)) {
            deviceInfoList.forEach(d -> {
                d.setProCode(proCode);
                // 更新设备状态
                this.saveAndUpdate(d);

            });
            // 更新产品设备数量
            productInfoMapper.updateEqTotal(deviceInfoList.get(0).getOrgCode(), proCode);
        }
    }

    @Override
    public List<DeviceInfo> proCodeFromOrgCodeAndLabelCode(String orgCode, String labelCode) {
        return deviceInfoMapper.proCodeFromOrgCodeAndLabelCode(orgCode, labelCode);
    }

}
