package com.unissoft.visualisation.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.unissoft.alarm.service.AlarmRecordService;
import com.unissoft.basic.model.AlarmParam;
import com.unissoft.basic.service.AlarmParamService;
import com.unissoft.common.DateUtils;
import com.unissoft.common.MyConstant;
import com.unissoft.deviceManage.model.DeviceData;
import com.unissoft.deviceManage.model.DevicePoint;
import com.unissoft.deviceManage.service.DeviceDataService;
import com.unissoft.visualisation.model.VisMonthWeek;
import com.unissoft.visualisation.service.VisualisationService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @author qingsong.han
 * @description:
 * @create 2022-10-17 10:40
 */
@Slf4j
@Service
public class VisualisationServiceImpl implements VisualisationService {
    @Resource
    private AlarmParamService alarmParamService;
    @Resource
    private DeviceDataService deviceDataService;
    @Resource
    private AlarmRecordService alarmRecordService;
    @Resource
    private PointChild pointChild;

    @Override
    public Map<String, List<AlarmParam>> alarmParamVis(String orgCode, String labelCode) {
        List<AlarmParam> list = alarmParamService.alarmParamVis(orgCode, labelCode);
        if (CollectionUtils.isEmpty(list))
            return Collections.emptyMap();
        else
            return list.stream().collect(Collectors.groupingBy(AlarmParam::getAlarmTypeCode));
    }

    @Override
    public PointChild initPointsDraw(String orgCode, String labelCode, Integer pageSize) {
        // 获取数据
        List<DeviceData> deviceData = deviceDataService.initPointsDraw(orgCode, labelCode, pageSize);
        // 缓存多个list
        List<DevicePoint> lp = new ArrayList<>();
        // 分组处理
        if (!CollectionUtils.isEmpty(deviceData)) {
            deviceData.forEach(d -> {
                String points = d.getPoints();
                if (!ObjectUtils.isEmpty(points)) {
                    List<DevicePoint> devicePoints = JSONArray.parseArray(points, DevicePoint.class);
                    // add all data,because Absence
                    devicePoints.forEach(dp -> {
                        dp.setTime(d.getDataTime());
                        dp.setOrgCode(d.getOrgCode());
                        // add list to object
                        lp.add(dp);
                    });
                }
            });
            // 返回可视化可用分组数据
            Map<String, List<DevicePoint>> pointListMap = lp.stream().collect(Collectors.groupingBy(DevicePoint::getDevice));
            pointChild.setLp(pointListMap);
            pointChild.setProCodes(deviceData.get(0).getProCode());
        }
        return pointChild;
    }

    @Override
    public Map<String, Object> monthAndWeek(String orgCode, String labelCode, Integer round) {
        // between time begin and end
        String begin;
        String end;
        // data
        List<VisMonthWeek> list;
        // result map
        Map<String, Object> map;
        switch (round) {
            // This year every month data
            case Calendar.MONTH:
                // First day in this year
                begin = DateUtils.joinTimeHMS(DateUtils.formatDate(DateUtils.getCurrentFirstOfYear()), DateUtils.startTime);
                // Last day in this year
                end = DateUtils.joinTimeHMS(DateUtils.formatDate(DateUtils.getCurrentLastOfYear()), DateUtils.endTime);
                list = deviceDataService.monthAndWeek(orgCode, labelCode, begin, end);
                // array init month
                double[] monthDraw = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
                // filter data
                map = this.filterData(list, monthDraw, null);
                break;
            case Calendar.DAY_OF_WEEK:
                Date date = DateUtils.getDate();
                // today in the week
                int week = DateUtils.dateToMonthOrWeek(date, Calendar.DAY_OF_WEEK);
                // 1 2 3 4 5 6 7
                begin = DateUtils.joinTimeHMS(
                        DateUtils.formatDate(DateUtils.getDate(date.getTime() - ((week - Calendar.SUNDAY) * MyConstant.ONE_DAY))),
                        DateUtils.startTime
                );
                end = DateUtils.joinTimeHMS(DateUtils.formatDate(date), DateUtils.endTime);
                list = deviceDataService.monthAndWeek(orgCode, labelCode, begin, end);
                // array init week
                double[] weekTempDraw = {0, 0, 0, 0, 0, 0, 0};
                double[] weekWetDraw = {0, 0, 0, 0, 0, 0, 0};
                // filter data
                map = this.filterData(list, weekTempDraw, weekWetDraw);
                break;
            default:
                map = Collections.emptyMap();
                break;
        }
        return map;
    }

    @Override
    public List<Map> alarmRecord(String orgCode, String labelCode, Integer pageSize) {
        return alarmRecordService.alarmRecordVis(orgCode, labelCode, pageSize);
    }

    // filer week and month data
    private Map<String, Object> filterData(List<VisMonthWeek> list, double[] firstFlower, double[] secondFlower) {
        if (!CollectionUtils.isEmpty(list)) {
            synchronized (this) {
                // Both month and week draws outside value
                Map<String, Object> map = new ConcurrentHashMap<>();
                // This is a max value of temperature to month draw
                Optional<VisMonthWeek> max = list.stream().max(Comparator.comparing(VisMonthWeek::getMaxValue));
                if (max.isPresent()) {
                    Double maxValue = max.get().getMaxValue();
                    map.put("maxTemp", maxValue);
                }
                // This is a min value of temperature to week draw
                Optional<VisMonthWeek> min = list.stream().min(Comparator.comparing(VisMonthWeek::getMinValue));
                if (min.isPresent()) {
                    Double minValue = min.get().getMinValue();
                    map.put("minTemp", minValue);
                }
                // convert datetime for month or week
                list.forEach(visMonthWeek -> {
                    int sort = DateUtils.dateToMonthOrWeek(
                            visMonthWeek.getDataTime(), secondFlower == null ? Calendar.MONTH : Calendar.DAY_OF_WEEK
                    );
                    visMonthWeek.setSort(sort);
                });
                // data group
                Map<Integer, List<VisMonthWeek>> collectMap = list.stream().collect(Collectors.groupingBy(VisMonthWeek::getSort));
                // month or week avg
                collectMap.forEach((k, v) -> {
                    // temperature average
                    OptionalDouble avgTemp = v.stream().mapToDouble(VisMonthWeek::getAvgValue).average();
                    if (avgTemp.isPresent()) {
                        firstFlower[k - 1] = avgTemp.getAsDouble();
                    }
                    // wet average
                    OptionalDouble avgWet = v.stream().mapToDouble(VisMonthWeek::getAvgWet).average();
                    if (avgWet.isPresent())
                        if (secondFlower != null)
                            secondFlower[k - 1] = avgWet.getAsDouble();
                });

                if (secondFlower != null) {
                    // wet average value to line draw
                    Optional<VisMonthWeek> wetMax = list.stream().max(Comparator.comparing(VisMonthWeek::getMaxWet));
                    if (wetMax.isPresent()) {
                        Double maxWet = wetMax.get().getMaxWet();
                        map.put("maxWet", maxWet);
                    }
                    map.put("avgWet", secondFlower);
                }
                map.put("avgTemp", firstFlower);
                return map;
            }
        }
        return Collections.emptyMap();
    }

}
